function  [y,v,a] = splin2(ym,dT,w)
%SPLIN2    2nd-order spline smoothing and numerical differentiation.
%
%   [y,v,a] = splin2(ym,dT,w)
%   
%  y    smoothed output
%  v    1st-order derivative
%  a    2nd-order derivative 
%  ym   row vector of measurements
%  dT   sample time
%  w    width of moving data window (w+1 samples)
%
%   * outputs a smoothed output and its 1st and 2nd-order derivative
%   * non-causal spline interpolation method, i.e.
%     estimates are calculated at the mid-point of the moving window
%   * the algorithm is initialized with estimates from the beginning 
%     of the moving window
   
% Version: (author: G-O K)
%  1.1 - 29/04/2003, rewritten layout.
%  1.0 - 06/08/2002, created.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% INITIALIZATION %%%
% Output vectors:
N=length(ym);
y=zeros(1,N);
v=zeros(1,N);
a=zeros(1,N);
% Least-squares problem matrix:
A=zeros(w+1,3);
for j=0:w
    A(j+1,1:3)=[1 j*dT (j*dT)^2];
end
% Window mid-point (k-d):
d=round(w/2);

%%% SPLINE FITTING FOR-LOOP %%%
for k=w+1:N
    % Data window with last sample k:
    b=ym(k-w:k)';
    % Calculates least-squares fit:
    x=A\b;
    if k==w+1
        % Estimates of the w-d first estimates:
        for j=1:w-d
            dt=(j-1)*dT;
            y(j)=[1 dt dt^2]*x;
            v(j)=[0 1 2*dt]*x;
            a(j)=[0 0 2]*x;
        end
    end
% Estimates at step k:
    dt=d*dT;
    y(k-d)=[1 dt dt^2]*x;
    v(k-d)=[0 1 2*dt]*x;
    a(k-d)=[0 0 2]*x;
    if k==N
        % Estimates of the d last estimates:
        for j=1:d
            dt=(d+j)*dT;
            y(k-d+j)=[1 dt dt^2]*x;
            v(k-d+j)=[0 1 2*dt]*x;
            a(k-d+j)=[0 0 2]*x;
        end
    end
end
